-- See the Copyright notice at the end of this file.
--
class STANDARD_STREAMS
	--
	-- Thanks to this `standard_streams' singleton object, you can redirect `io', `std_input', `std_output' as
	-- well as `std_error'. (See also examples from our tutorial/io directory.)
	--

inherit
	ANY
		rename io as any_io,
			std_input as any_std_input,
			std_output as any_std_output,
			std_error as any_std_error
		end

creation {ANY}
	make

feature {ANY}
	io: STD_INPUT_OUTPUT

	std_input: INPUT_STREAM

	std_output: OUTPUT_STREAM

	std_error: OUTPUT_STREAM

feature {ANY}
	set_std_input (a_std_input: like std_input) is
		require
			a_std_input.is_connected
		do
			std_input := a_std_input
		end

	restore_std_input is
		do
			if system_std_input.is_filtered then
				system_std_input.detach
			end
			set_std_input(system_std_input)
		end

	set_std_output (a_std_output: like std_output) is
		require
			a_std_output.is_connected
		do
			std_output := a_std_output
		end

	restore_std_output is
		do
			if system_std_output.is_filtered then
				system_std_output.detach
			end
			set_std_output(system_std_output)
		end

	set_std_error (a_std_error: like std_error) is
		require
			a_std_error.is_connected
		do
			std_error := a_std_error
		end

	restore_std_error is
		do
			set_std_error(system_std_error)
			if system_std_error.is_filtered then
				system_std_error.detach
			end
		end

feature {ANY}
	system_std_input: STD_INPUT

	system_std_output: STD_OUTPUT

	system_std_error: STD_ERROR

feature {}
	make is
		do
			create system_std_input.make
			create system_std_output.make
			create system_std_error.make
			create io.make
			restore_std_input
			restore_std_output
			restore_std_error
		end

feature {}
	frozen singleton: STANDARD_STREAMS is
		once
			Result := Current
		end

invariant
	Current = singleton

	std_input /= Void

	std_output /= Void

	std_error /= Void

	io /= Void

end -- class STANDARD_STREAMS
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
