-- See the Copyright notice at the end of this file.
--
class STD_INPUT_OUTPUT
	--
	-- Just a centralized access to `std_input' and `std_output', because it's
	-- quite useful (good old `io').
	--
	-- Note that it cannot be filtered itself (see invariant). Instead,
	-- use STANDARD_STREAMS features to modify std_input and std_output.
	--

inherit
	TERMINAL_INPUT_OUTPUT_STREAM

creation {STANDARD_STREAMS}
	make

feature {}
	make is
		do
		end

feature {ANY}
	is_connected: BOOLEAN is
		do
			-- Yes, it is an "and" and not an "or": it must be strong enough,
			-- otherwise the system won't work
			Result := std_input /= Void and then std_input.is_connected and then (std_output /= Void and then std_output.is_connected)
		end

	disconnect is
		do
			std_input.disconnect
			std_output.disconnect
			standard_streams.restore_std_input
			standard_streams.restore_std_output
			input_filter := Void
			output_filter := Void
		end

feature {ANY}
	end_of_input: BOOLEAN is
		do
			Result := std_input.end_of_input
		end

	can_unread_character: BOOLEAN is
		do
			Result := std_input.can_unread_character
		end

feature {FILTER_INPUT_STREAM}
	filtered_read_character is
		do
			std_input.read_character
		end

	filtered_unread_character is
		do
			std_input.unread_character
		end

	filtered_last_character: CHARACTER is
		do
			Result := std_input.last_character
		end

feature {FILTER_OUTPUT_STREAM}
	filtered_put_character (c: CHARACTER) is
		do
			std_output.put_character(c)
		end

	filtered_flush is
		do
			std_output.flush
		end

feature {FILTER}
   filtered_descriptor: INTEGER is
      do
      end

   filtered_has_descriptor: BOOLEAN is False

feature {}
	singleton: STD_INPUT_OUTPUT is
		once
			Result := Current
		end

invariant
	is_singleton: singleton = Current

	not_filtered: not input_is_filtered and not output_is_filtered

end -- class STD_INPUT_OUTPUT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
