-- See the Copyright notice at the end of this file.
--
class STD_OUTPUT
	--
	-- To use the standard output file. As for UNIX, the default standard
	-- output is the screen.
	--
	-- Notes: - the predefined `std_output' should be use to have only one instance
	--        of the class STD_OUTPUT,
	--        - to do reading or writing at the same time on the screen,
	--        see STD_INPUT_OUTPUT,
	--
	-- See also STANDARD_STREAMS
	--

inherit
	TERMINAL_OUTPUT_STREAM

creation {STANDARD_STREAMS}
	make

feature {ANY}
	is_connected: BOOLEAN is True

	disconnect is
		do
			filter := Void
		end

feature {}
	make is
		do
			buffer := buffer.calloc(4096)
			capacity := 4096
		end

feature {FILTER_OUTPUT_STREAM}
	filtered_put_character (c: CHARACTER) is
		do
			if buffer_position >= 4096 then
				write_buffer
			end
			buffer.put(c, buffer_position)
			buffer_position := buffer_position + 1
			if c = '%N' then
				write_buffer
			end
		end

	filtered_flush is
		do
			if buffer_position > 0 then
				write_buffer
			end
			io_flush(stdout)
		end

feature {FILTER}
   filtered_descriptor: INTEGER is
      do
         Result := basic_file_descriptor(stdout)
      end

   filtered_has_descriptor: BOOLEAN is True

feature {STD_INPUT}
	buffer_position: INTEGER

feature {}
	buffer: NATIVE_ARRAY[CHARACTER]

	capacity: INTEGER

	write_buffer is
		do
			if buffer_position > 0 then
				io_fwrite(buffer, buffer_position, stdout)
				buffer_position := 0
			end
		end

	stdout: POINTER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "io"
	 feature_name: "stdout"
	 }"
		end

end -- class STD_OUTPUT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
