-- See the Copyright notice at the end of this file.
--
class STRING_OUTPUT_STREAM
	--
	-- An output stream where data is written to a string buffer.
	--

inherit
	TERMINAL_OUTPUT_STREAM

creation {ANY}
	make

feature {ANY}
	to_string: STRING is
			-- A copy of the internal buffer. If you don't want memory consumption, see `append_in'.
		do
			Result := string.twin
		end

	append_in (a_string: STRING) is
			-- Append the contents of the internal buffer to the given string.
		require
			a_string /= Void
		do
			a_string.append(string)
		end

	is_connected: BOOLEAN is True

	disconnect is
		do
			filter := Void
		end

	clear is
			-- Remove all stored characters
		do
			string.clear_count
		ensure
			string.is_empty
		end

feature {FILTER_OUTPUT_STREAM}
	filtered_put_character (c: CHARACTER) is
		do
			string.extend(c)
		end

	filtered_flush is
		do
			-- nothing to do
		end

feature {FILTER}
   filtered_descriptor: INTEGER is
      do
      end

   filtered_has_descriptor: BOOLEAN is False

feature {}
	make is
		do
			string := ""
		end

	string: STRING
			-- where the characters go to

end -- class STRING_OUTPUT_STREAM
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
