-- See the Copyright notice at the end of this file.
--
class WEAK_REFERENCE[G]
	--
	-- Weak reference to some object of type G.
	-- This kind of reference does not prevent the object G from being reclaimed by the garbage
	-- collector (in which case item returns Void).
	-- Item makes it possible to get (a strong reference to) the object.
	-- Inheriting from this class is prohibited.
	--
	-- Notes: when using the "-no_gc" option (i.e. no garbage collector), the behavior of WEAK_REFERENCE[G] 
	-- is equivalent to the behavior of REFERENCE[G]. Also note that WEAK_REFERENCE[G] is not yet implemented 
	-- for `compile_to_jvm' (the behavior is the one of REFERENCE[G]).
	--

inherit
	ANY

creation {ANY}
	set_item

feature {ANY}
	item: G
			-- Return a (strong) reference to the object

	set_item (i: like item) is
			-- Set the object to be weak referenced
		do
			item := i
		ensure
			item = i
		end

end -- class WEAK_REFERENCE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
