class SOCKET
--
-- A low-level socket. Should not be used directly, but only via a SOCKET_INPUT_OUTPUT_STREAM.
--
inherit
   DISPOSABLE

insert
   STRING_HANDLER
   SOCKET_HANDLER
   SOCKET_PLUG_IN

create {SOCKET_HANDLER}
   make

create {SOCKET_SERVER}
   bind

feature {SOCKET_HANDLER}
   connect_tcp (a_port: INTEGER) is
      do
         port := a_port
         connect(net_tcp(a, b, c, d, a_port))
      end

   connect_udp (a_port: INTEGER) is
      do
         port := a_port
         connect(net_udp(a, b, c, d, a_port))
      end

   connect_local (a_port: INTEGER) is
      do
         port := a_port
         connect(net_local(a_port))
      end

   is_connected: BOOLEAN

   read (timeout: REAL) is
      require
         is_connected
      local
         count: INTEGER
      do
         if net_select(fd, timeout) = fd then
            last_read.clear_count
            from
               count := read_more
            until
               count < default_buffer_size or else last_error /= Void
            loop
               count := read_more
            end
         end
         if last_error /= Void then
            disconnect
         end
      end

   last_read: STRING

   write (data: STRING) is
      require
         is_connected
         data /= Void
      local
         dummy: INTEGER
      do
         dummy := net_write(fd, data.count, data.storage)
         if last_error /= Void then
            disconnect
         end
      end

   disconnect is
      do
         if is_connected then
            net_disconnect(fd)
            is_connected := False
         end
      ensure
         not is_connected
      end

   a, b, c, d: INTEGER
   port: INTEGER

   fd: INTEGER

feature {SOCKET_SERVER}
   bind (server: SOCKET_SERVER) is
      require
         not is_connected
      do
         if bind_values.is_null then
            bind_values := bind_values.calloc(6)
         end
         net_accept(server.fd, bind_values)
         fd := bind_values.item(5)
         if fd >= 0 then
            check
               last_error = Void
            end
            make(bind_values.item(0), bind_values.item(1), bind_values.item(2), bind_values.item(3))
            port := bind_values.item(4)
            is_connected := True
         end
      end

feature {}
   make (ip_a, ip_b, ip_c, ip_d: INTEGER) is
      do
         a := ip_a
         b := ip_b
         c := ip_c
         d := ip_d
         buffer := buffer.calloc(default_buffer_size)
         last_read := ""
      end

   dispose is
      require
         should_not_rely_on_gc: not is_connected
      do
         disconnect
      end

   read_more: INTEGER is
      local
         i: INTEGER
      do
         Result := net_read(fd, default_buffer_size, buffer)
         if Result > 0 then
            i := last_read.count
            last_read.resize(i + Result)
            last_read.storage.copy_at(i, buffer, Result)
         end
      end

   connect (a_fd: like fd) is
      do
         is_connected := last_error = Void and then a_fd >= 0
         fd := a_fd
      end

feature {} -- really low level
   buffer: NATIVE_ARRAY[CHARACTER]
   bind_values: NATIVE_ARRAY[INTEGER]

   default_buffer_size: INTEGER is 8192

end
