class SOCKET_INPUT_OUTPUT_STREAM
--
-- A stream backed by a socket. You cannot directly create such a stream; instead, use ADDRESS.stream for
-- an outbound connection; for an inbound connection, the stream is automatically created by the
-- SOCKET_SERVER and passed to its handlers.
--
inherit
   TERMINAL_INPUT_OUTPUT_STREAM

insert
   SOCKET_HANDLER

create {ADDRESS}
   connect_to

create {SOCKET_SERVER}
   connect_to_server

feature {ANY}
	is_connected: BOOLEAN is
      do
         Result := socket.is_connected
		end

	can_unread_character: BOOLEAN is
      do
         Result := index > socket.last_read.lower
		end

	end_of_input: BOOLEAN is
      do
         Result := not is_connected or else end_of_stream
      end

   disconnect is
      do
         if server /= Void then
            server.unbind
         end
         socket.disconnect
      end

   set_timeout (a_timeout: like timeout) is
      require
         a_timeout >= 0.0
      do
         timeout := a_timeout
      end

   unset_timeout is
         -- Makes the socket wait until some characters are available (no timeout)
      do
         timeout := no_timeout
      end

   try_again is
         -- Attempt another read. Useful when timeout is set.
      require
         is_connected
         end_of_input
      do
         ensure_read
      end

feature {FILTER_INPUT_STREAM}
	filtered_read_character is
      do
         index := index + 1
         ensure_read
		end

	filtered_unread_character is
      do
         index := index - 1
		end

	filtered_last_character: CHARACTER is
      do
         Result := socket.last_read.item(index)
		end

feature {FILTER_OUTPUT_STREAM}
	filtered_put_character (c: CHARACTER) is
      do
         out_buffer.add_last(c)
         if c = '%N' then
            filtered_flush
         end
		end

	filtered_flush is
      do
         socket.write(out_buffer)
         out_buffer.clear_count
		end

feature {FILTER}
   filtered_descriptor: INTEGER is
      do
         Result := socket.fd
      end

   filtered_has_descriptor: BOOLEAN is True

feature {}
   ensure_read is
         -- Read some new data from the socket if it is available. Does nothing if all the already read data
         -- is not yet consumed.
      do
         if socket.is_connected then
            if socket.last_read.valid_index(index) then
               end_of_stream := False
            else
               socket.read(timeout)
               end_of_stream := not socket.is_connected or else socket.last_read.is_empty
               index := 1
            end
         else
            end_of_stream := True
         end
      end

   end_of_stream: BOOLEAN

feature {}
   connect_to (a_socket: SOCKET) is
      require
         a_socket /= Void
      do
         socket := a_socket
         out_buffer := ""
         unset_timeout
      end

   connect_to_server(a_server: SOCKET_SERVER) is
      require
         a_server /= Void
      do
         server := a_server
         connect_to(a_server.bind)
      end

   socket: SOCKET

   server: SOCKET_SERVER

   index: INTEGER

   timeout: REAL

   out_buffer: STRING

   no_timeout: REAL is -1.0

invariant
   socket /= Void

end
