class SOCKET_SERVER
--
-- A socket server waits for connections to happen on some given address/port. It then defers treatment to
-- handlers (see the `when_connect' feature).
--
-- Please use ADDRESS.server to create a SOCKET_SERVER.
--
inherit
   JOB

insert
   SOCKET_HANDLER

creation {SOCKET_HANDLER}
   make

feature {ANY}
   when_connect (handler: PROCEDURE[ANY, TUPLE[SOCKET_INPUT_OUTPUT_STREAM]]) is
      require
         can_connect
      do
         handlers.add_last(handler)
      end

   can_connect: BOOLEAN is
      do
         Result := not done
      end

feature {LOOP_ITEM}
	prepare (ready: READY_DESCRIPTION) is
      do
         if not done and then fd >= 0 then
            ready.when_connection(Current)
         end
		end

	is_ready (ready: READY_DESCRIPTION): BOOLEAN is
		do
         -- is there an incoming connection?
         Result := not done and then fd >= 0 and then ready.is_connection(Current)
		end

	continue is
      local
         stream: SOCKET_INPUT_OUTPUT_STREAM
         i, n: INTEGER
      do
         -- on incoming connection only (see `is_ready')
         create stream.connect_to_server(Current)
         check
            stream.is_connected
         end
         from
            i := handlers.lower
            n := handlers.upper
         until
            i > n
         loop
            handlers.item(i).call([stream])
            i := i + 1
         end
		end

	done: BOOLEAN

	restart is
      do
         -- nothing (yet?)
		end

feature {SOCKET_HANDLER}
   a, b, c, d: INTEGER
   access: ACCESS

feature {READY_DESCRIPTION, SOCKET_HANDLER}
   fd: INTEGER

feature {SOCKET_INPUT_OUTPUT_STREAM}
   bind: SOCKET is
      do
         Result := binding
         if Result = Void then
            create Result.bind(Current)
            binding := Result
         elseif not Result.is_connected then
            Result.bind(Current)
         end
         done := not Result.is_connected
      end

   unbind is
      do
         done := True
         if binding /= Void then
            binding.disconnect
         end
      end

feature {}
   binding: SOCKET

feature {}
   make (ip_a, ip_b, ip_c, ip_d: INTEGER; a_access: ACCESS) is
      do
         a := ip_a
         b := ip_b
         c := ip_c
         d := ip_d
         access := a_access
         fd := a_access.server_socket
         done := (fd < 0)
         create handlers.with_capacity(2)
      end

   handlers: FAST_ARRAY[PROCEDURE[ANY, TUPLE[SOCKET_INPUT_OUTPUT_STREAM]]]

end
