-- See the Copyright notice at the end of this file.
--
deferred class GEN_RAND
	--
	-- Here is the common way to use a random number generator.
	-- Current implementations are MIN_STAND, STD_RAND.
	--

feature {ANY} -- Creation procedures:
	make is
			-- Create the generator with an automatic hazardous setting of
			-- the `seed_value'.
			-- Because automatic setting may be done using internal address
			-- of Current for example, it may produces platform dependent
			-- behavior or compilation-mode dependant behavior.
			-- Also consider `with_seed' to chose the most appropriate.
		deferred
		end

	with_seed (seed_value: INTEGER) is
			-- Create the generator with an explicit `seed_value'.
		deferred
		end

feature {ANY}
	next is
			-- Compute next random number in sequence.
		deferred
		end

feature {ANY} -- No modifications:
	last_double: REAL is
			-- Look at the last computed number.
			-- Range 0 to 1
		obsolete "Use `last_real' instead (nov 4th 2004)."
		do
			Result := last_real
		ensure
			Result > 0 and Result <= 1
		end

	last_real: REAL is
			-- Look at the last computed number.
			-- Range 0 to 1
		deferred
		ensure
			Result > 0 and Result <= 1
		end

	last_integer (n: INTEGER): INTEGER is
			-- Look the last computed number.
			-- Range 1 to `n'.
		require
			n >= 1
		deferred
		ensure
			1 <= Result and Result <= n
		end

end -- class GEN_RAND
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
