-- See the Copyright notice at the end of this file.
--
class LOOP_ITEM
	-- One loop level with related jobs to run

creation {LOOP_STACK}
	make

feature {LOOP_STACK}
	job_list: FAST_ARRAY[JOB]

feature {}
	finished_jobs: FAST_ARRAY[JOB]

	ready_jobs: FAST_ARRAY[JOB]

	ready: READY_DESCRIPTION

	make is
		do
			create job_list.make(0)
			create finished_jobs.make(0)
			create ready_jobs.make(0)
			create ready.make
			pause := True
		ensure
			pause
			not break
		end

feature {LOOP_STACK}
	pause: BOOLEAN

	break: BOOLEAN

	run is
		require
			pause and not break
		local
			i: INTEGER; selected_priority: INTEGER
		do
			from
				pause := False
				break := False
			until
				pause or else break or else job_list.is_empty
			loop
				-- prepare jobs
				from
					i := job_list.lower
					ready.reset
					ready.current_time_update
				until
					i > job_list.upper
				loop
					job_list.item(i).prepare(ready)
					i := i + 1
				end
				-- wait something to do
				ready.wait
				-- search ready jobs (with same priority as the first ready job)
				from
					i := job_list.lower
					ready.current_time_update
					selected_priority := Minimum_integer
					ready_jobs.clear_count
				until
					i > job_list.upper or else selected_priority /= Minimum_integer
				loop
					if job_list.item(i).is_ready(ready) then
						selected_priority := job_list.item(i).priority
						ready_jobs.add_last(job_list.item(i))
					end
					i := i + 1
				end
				from
				until
					i > job_list.upper or else job_list.item(i).priority /= selected_priority
				loop
					if job_list.item(i).is_ready(ready) then
						ready_jobs.add_last(job_list.item(i))
					end
					i := i + 1
				end
				-- run selected ready jobs
				from
					i := ready_jobs.upper
				until
					i < ready_jobs.lower
				loop
					ready_jobs.item(i).continue
					if ready_jobs.item(i).done then
						job_list.remove(job_list.fast_index_of(ready_jobs.item(i)))
						finished_jobs.add_last(ready_jobs.item(i))
					end
					i := i - 1
				end
			end
		ensure
			pause or job_list.is_empty or break
		end

	break_loop is
		do
			break := True
		ensure
			break
		end

	pause_loop is
		require
			not pause
		do
			pause := True
		ensure
			pause
		end

	add_job (j: JOB) is
		require
			j /= Void
		local
			i: INTEGER
		do
			from
				i := job_list.lower
			until
				i > job_list.upper or else job_list.item(i).priority > j.priority
			loop
				i := i + 1
			end
			job_list.add(j, i)
		end

	restart is
		require
			not pause or break
		do
			from
			until
				finished_jobs.is_empty
			loop
				finished_jobs.last.restart
				add_job(finished_jobs.last)
				finished_jobs.remove_last
			end
			pause := True
			break := False
		ensure
			pause
			not break
		end

end -- class LOOP_ITEM
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
