-- See the Copyright notice at the end of this file.
--
expanded class COMPARATOR_COLLECTION_SORTER[X]
	--
	-- Some algorithms to sort any COLLECTION using an external comparator.
	--
	-- Elements are sorted using the order given by the comparator: large elements at the beginning of the collection, small at the
	-- end (increasing order is implemented by class COLLECTION_SORTER).
	--
	-- Note that without setting a comparator (using `set_comparator'), collections won't get sorted.
	--
	-- How to use this expanded class :
	--
	--          local
	--             sorter: COMPARATOR_COLLECTION_SORTER[INTEGER]
	--             array: ARRAY[INTEGER]
	--          do
	--             array := <<1,3,2>>
	--             sorter.set_comparator(agent my_comparator)
	--             sorter.sort(array)
	--             check
	--                sorter.is_sorted(array)
	--             end
	--             ...
	--

inherit
	ABSTRACT_SORTER[X]
		redefine default_create
		end

creation {ANY}
	default_create, with_comparator

feature {ANY}
	set_comparator (a_comparator: like comparator) is
		do
			comparator := a_comparator
		end

	comparator: PREDICATE[ANY, TUPLE[X, X]]

feature {}
	lt (x, y: X): BOOLEAN is
		do
			Result := comparator.item([x, y])
		end

	default_comparator (x, y: X): BOOLEAN is
		do
			Result := True -- does not sort by default (sort algorithms are conservative)
		end

	default_create is
		do
			set_comparator(agent default_comparator)
		end

	with_comparator (a_comparator: like comparator) is
		do
			set_comparator(a_comparator)
		end

end -- class COMPARATOR_COLLECTION_SORTER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
