-- See the Copyright notice at the end of this file.
--
class BASIC_FONT
	-- BASIC_FONT describe basic font properties (like size, font family...)
	-- and allow to draw characters (see class `FONT' for options like
	-- underline).
	-- Use `FONT_MANAGER' for `BASIC_FONT' creation.
	--

inherit
	GRAPHIC
	UNICODE_STRING_HANDLER

creation {ANY}
	make_system_specific

creation {FONT_MANAGER}
	make_from_id

feature {} --   name: STRING
	make_system_specific (font_name: STRING) is
			-- It's recommended not to use this function.
			-- The `font_name' is the font name in the system syntax (ex: XLFD
			-- for X11).
		require
			font_name /= Void
			font_manager.font_exist_by_name(font_name)
		do
			make_from_id(basic_font_new(font_name))
		end

	make_from_id (font_id: POINTER) is
		do
			font := font_id
			font_desc := basic_font_properties(font)
			height := basic_font_height(font_desc)
			base_line := basic_font_ascent(font_desc)
		end

feature {ANY}
	height: INTEGER

	base_line: INTEGER -- height from the top to the bottom of characters like 'P' or 'k'. Height is bigger due to characters that go under the base line like 'j' or 'p'.

	text_width (text: UNICODE_STRING): INTEGER is
		do
			--TODO: surrogate characters !
			--TODO: combining characters !
			Result := basic_font_text_width(font_desc, font, text.storage.to_external, text.count)
		end

feature {BASIC_FONT, DRAW_STYLE}
	font: POINTER

	font_desc: POINTER --TODO: suppress this pointer

feature {}
	basic_font_properties (font_id: POINTER): POINTER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_properties"
	 }"
		end
		--   c_inline_c("{GContext gc;XFontStruct *font;%N%
		--	      %gc=XGContextFromGC(DefaultGC(dpy,DefaultScreen(dpy)));%N%
		--	      %font=XQueryFont(dpy,gc); R = font;}%N");
		--default_font_name: STRING is "-adobe-times-medium-r-normal--14-*-*-*-*-*-iso8859-1"
		--default_font_name: STRING is "fixed" --*-*-*-*-*-*-*-*-*-*-*-*-*-*"
		--default_font_name: STRING is "-adobe-times-*-i-*-*-*-300-*-*-*-*-iso8859-1"

	default_font_name: STRING is "-*-clearlyu-*-*-*-*-*-300-*-*-p-*-iso10646-1"
			--default_font_name: STRING is "-*-charter-*-*-*-*-*-500-*-*-*-*-iso8859-1"
			--default_font_name: STRING is "-monotype-corsiva-*-*-*-*-80-*-100-100-*-*-iso10646-*"
			--default_font_name: STRING is "-gnu-*-*-*-*-*-16-*-75-*-*-*-iso10646-*"

	basic_font_new (font_name: STRING): POINTER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_new"
	 }"
		end

	basic_font_height (f: POINTER): INTEGER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_height"
	 }"
		end

	basic_font_ascent (f: POINTER): INTEGER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_ascent"
	 }"
		end

	basic_font_text_width (fdes, fon: POINTER; str: POINTER; size: INTEGER): INTEGER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_text_width"
	 }"
		end

end -- class BASIC_FONT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
