-- See the Copyright notice at the end of this file.
--
class FONT_MANAGER
	-- The font manager is singleton accessible via `font_manager'
	-- from `GRAPHIC'. This class help to find the font on the system best
	-- matching given whishes.
	-- All created fonts are accessible in `fonts' attribute, the
	-- default font on the system is the first one.

creation {VISION}
	init

feature {}
	init is
		local
			f: BASIC_FONT; tmp: STRING
		do
			create fonts.make(0)
			create tmp.from_external(basic_font_default_name)
			create f.make_from_id(basic_font_new(tmp.to_external))
			fonts.add_last(f)
		end

feature {ANY}
	fonts: FAST_ARRAY[BASIC_FONT]
			-- All allocated fonts, the first one is the default system font.

	default_font: BASIC_FONT is
			-- The default system font, always defined.
		do
			Result := fonts.first
		ensure
			Result /= Void
		end

	font_exist_by_name (font_name: STRING): BOOLEAN is
			-- It's recommended not to use this function.
			-- The `font_name' is the font name in the system syntax (ex: XLFD
			-- for X11).
		require
			font_name /= Void
		do
			Result := basic_font_exist(font_name.to_external)
		end

feature {}
	basic_font_default_name: POINTER is
			-- should return the font id and suppress folowing function
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_default_name"
	 }"
		end

	basic_font_new (font_name: POINTER): POINTER is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_new"
	 }"
		end

	basic_font_exist (desc: POINTER): BOOLEAN is
		external "plug_in"
		alias "{
	 location: "${sys}runtime"
	 module_name: "basic_font"
	 feature_name: "basic_font_exist"
	 }"
		end

end -- class FONT_MANAGER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
