class BIJECTIVE_DICTIONARY[V -> HASHABLE, K -> HASHABLE]
-- ***
-- This class will be part of the SmartEiffel/lib/storage only for release 2.2 (actually
-- this will become 2 classes BIJECTIVE_DICTIONARY and HASHED_BIJECTIVE_DICTIONARY).
-- This is here just for compiler usage at time beeing. *** Dom feb 7th 2005 ***
-- ***
	
creation
	make
	
feature {ANY} -- Counting:
	count: INTEGER
			-- Actual `count' of stored elements.
	
	is_empty: BOOLEAN is
			-- Is it empty ?
		do
			Result := count = 0
		ensure
			Result = (count = 0)
		end

feature {ANY} -- Basic access:
	has (k: K): BOOLEAN is
			-- Is there a value currently associated with key `k'?
		require
			k /= Void
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
			until
				node = Void or else key_safe_equal(node.key, k)
			loop
				node := node.next_key
			end
			Result := node /= Void
		ensure
			Result implies has_value(at(k))
		end

	at (k: K): V is
			-- Return the value associated to key `k'.
			-- (See also `reference_at' if V is a reference type.)
		require
			has(k)
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
			until
				key_safe_equal(node.key, k)
			loop
				node := node.next_key
			end
			Result := node.val
		ensure
			k.is_equal(key_at(Result))
		end

	reference_at (k: K): V is
			-- Return Void or the value associated with key `k'. Actually, this
			-- feature is useful when the type of values (the type V) is a
			-- reference type, to avoid using `has' just followed by `at' to get
			-- the corresponding value.
		require
			k /= Void
			values_are_expanded: Result /= Void implies not Result.is_expanded_type
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
			until
				node = Void or else key_safe_equal(node.key, k)
			loop
				node := node.next_key
			end
			if node /= Void then
				Result := node.val
			end
		ensure
			has(k) implies Result = at(k)
		end

	fast_has (k: K): BOOLEAN is
			-- Is there a value currently associated with key `k'?
		require
			k /= Void
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
			until
				node = Void or else node.key = k
			loop
				node := node.next_key
			end
			Result := node /= Void
		ensure
			Result implies fast_has_value(fast_at(k))
		end

	fast_at (k: K): V is
			-- Return the value associated to key `k'.
			-- (See also `reference_at' if V is a reference type.)
		require
			fast_has(k)
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
			until
				node.key = k
			loop
				node := node.next_key
			end
			Result := node.val
		ensure
			Result = at(k)
			fast_key_at(Result) = k
		end

	fast_reference_at (k: K): V is
			-- Return Void or the value associated with key `k'. Actually, this
			-- feature is useful when the type of values (the type V) is a
			-- reference type, to avoid using `has' just followed by `at' to get
			-- the corresponding value.
		require
			k /= Void
			values_are_expanded: Result /= Void implies not Result.is_expanded_type		
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
			until
				node = Void or else node.key = k
			loop
				node := node.next_key
			end
			if node /= Void then
				Result := node.val
			end
		ensure
			fast_has(k) implies Result = fast_at(k)
		end

	has_value (v: V): BOOLEAN is
			-- Is there a value `v'?
		require
			v /= Void
		local
			node: like cache_node
		do
			from
				node := val_buckets.item(v.hash_code #\\ capacity)
			until
				node = Void or else val_safe_equal(node.val, v)
			loop
				node := node.next_val
			end
			Result := node /= Void
		ensure
			Result implies has(key_at(v)) 
		end

	key_at (v: V): K is
			-- Retrieve the key used for value `v' using `equal' for comparison.
		require
			has_value(v)
		local
			node: like cache_node
		do
			from
				node := val_buckets.item(v.hash_code #\\ capacity)
			until
				val_safe_equal(node.val, v)
			loop
				node := node.next_val
			end
			Result := node.key
		ensure
			v.is_equal(at(Result))
		end

	fast_has_value (v: V): BOOLEAN is
			-- Is there a value `v'?
		require
			v /= Void
		local
			node: like cache_node
		do
			from
				node := val_buckets.item(v.hash_code #\\ capacity)
			until
				node = Void or else node.val = v
			loop
				node := node.next_val
			end
			Result := node /= Void
		ensure
			Result implies fast_has(fast_key_at(v))
		end

	fast_key_at (v: V): K is
			-- Retrieve the key used for value `v' using `=' for comparison.
		require
			fast_has_value(v)
		local
			node: like cache_node
		do
			from
				node := val_buckets.item(v.hash_code #\\ capacity)
			until
				node.val = v
			loop
				node := node.next_val
			end
			Result := node.key
		ensure
			Result = key_at(v)
			fast_at(Result) = v
		end

feature {ANY}
	put (v: V; k: K) is
			-- Change some existing entry or `add' the new one. If there is
			-- as yet no key `k' in the dictionary, enter it with item `v'.
			-- Otherwise overwrite the item associated with key `k'.
		require
			has(k) implies key_at(at(k)).is_equal(k)
			has_value(v) implies key_at(v).is_equal(k)
		do
			not_yet_implemented
		ensure
			value_updated: v = fast_at(k)
			key_updated: k = fast_key_at(v)
		end

	add (v: V; k: K) is
			-- To add a new entry `k' with its associated value `v'. Actually, this is equivalent to call 
			-- `put', but may run a little bit faster.
		require
			not has_value(v)
			not has(k)
		local
			key_idx, val_idx: INTEGER; node: like cache_node
		do
			cache_user := -1
			if capacity = count then
				increase_capacity
			end
			key_idx := k.hash_code #\\ capacity
			val_idx := v.hash_code #\\ capacity
			node := new_node(v, val_buckets.item(val_idx), k, key_buckets.item(key_idx))
			key_buckets.put(node, key_idx)
			val_buckets.put(node, val_idx)
			-- Finally:
			count := count + 1
		ensure
			count = 1 + old count
			v = fast_at(k)
			k = fast_key_at(v)
		end

feature {ANY} -- To provide iterating facilities:
	lower: INTEGER is 1

	upper: INTEGER is
		do
			Result := count
		ensure
			Result = count
		end

	valid_index (index: INTEGER): BOOLEAN is
		do
			Result := 1 <= index and then index <= count
		ensure
			Result = index.in_range(lower, upper)
		end

	item (index: INTEGER): V is
		require
			valid_index(index)
		do
			set_cache_user(index)
			Result := cache_node.val
		ensure
			Result = at(key(index))
		end

	key (index: INTEGER): K is
		require
			valid_index(index)
		do
			set_cache_user(index)
			Result := cache_node.key
		ensure
			Result = key_at(item(index))
		end

feature {ANY} -- Other features:
	Default_size: INTEGER is 193
			-- Default size for the storage area in number of items.

	internal_key (k: K): K is
			-- Retrieve the internal key object which correspond to the existing
			-- entry `k' (the one memorized into the `Current' dictionary).
		require
			has(k)
		local
			node: like cache_node
		do
			from
				node := key_buckets.item(k.hash_code #\\ capacity)
				Result := node.key
			until
				key_safe_equal(Result, k)
			loop
				node := node.next_key
				Result := node.key
			end
		ensure
			Result.is_equal(k)
			internal_key(Result) = Result
			at(k) = fast_at(Result)
		end

feature {BIJECTIVE_DICTIONARY}
	key_buckets: NATIVE_ARRAY[like cache_node]
			-- The `key_buckets' storage area is the primary hash table of `capacity' elements. To search some 
			-- key, the first access is done in `key_buckets' using the remainder of the division of the key 
			-- `hash_code' by `capacity'. In order to try to avoid clashes, `capacity' is always a prime 
			-- number (selected using HASH_TABLE_SIZE).

	val_buckets: NATIVE_ARRAY[like cache_node]
			-- The `val_buckets' storage area is the primary hash table of `capacity' elements. To search some 
			-- value, the first access is done in `val_buckets' using the remainder of the division of the value 
			-- `hash_code' by `capacity'. In order to try to avoid clashes, `capacity' is always a prime 
			-- number (selected using HASH_TABLE_SIZE).

	capacity: INTEGER
			-- Approximation of the actual internal storage `capacity'. The `capacity' will grow automatically
			-- when needed (i.e. `capacity' is not a limit for the number of values stored). Also note that
			-- the `capacity' value may not be always accurate depending of the implementation (anyway, this
			-- `capacity' value is at least equals to `count').

feature {}
	cache_user: INTEGER
			-- The last user's external index in range [1 .. `count'] (see `item' and `valid_index' for example) 
			-- may be saved in `cache_user' otherwise -1 to indicate that the cache is not active. When the 
			-- cache is active, the corresponding index in `key_buckets' is save in `cache_buckets' and the 
			-- corresponding node in `cache_node'.

	cache_node: HASHED_BIJECTIVE_DICTIONARY_NODE[V, K]
			-- Meaningful only when `cache_user' is not -1.

	cache_buckets: INTEGER
			-- Meaningful only when `cache_user' is not -1.

	set_cache_user (index: INTEGER) is
			-- Set the internal memory cache (`cache_user', `cache_node' and `cache_buckets') to the appropriate 
			-- valid value.
		require
			valid_index(index)
		do
			if index = cache_user + 1 then
				from
					cache_user := index
					cache_node := cache_node.next_key
				until
					cache_node /= Void
				loop
					cache_buckets := cache_buckets + 1
					cache_node := key_buckets.item(cache_buckets)
				end
			elseif index = cache_user then
			elseif index = 1 then
				from
					cache_user := 1
					cache_buckets := 0
					cache_node := key_buckets.item(cache_buckets)
				until
					cache_node /= Void
				loop
					cache_buckets := cache_buckets + 1
					cache_node := key_buckets.item(cache_buckets)
				end
			else
				from
					set_cache_user(1)
				until
					cache_user = index
				loop
					set_cache_user(cache_user + 1)
				end
			end
		ensure
			cache_user = index
			cache_buckets.in_range(0, capacity - 1)
			cache_node /= Void
		end

	make is
			-- Create an empty dictionary. Internal storage `capacity' of the dictionary is initialized using 
			-- the `Default_size' value. Then, tuning of needed storage `capacity' is performed automatically
			-- according to usage. If you are really sure that your dictionary is always really bigger than 
			-- `Default_size', you may consider to use `with_capacity' to save some execution time.
		do
			with_capacity(Default_size)
		ensure
			capacity = Default_size
		end

	with_capacity (medium_size: INTEGER) is
			-- May be used to save some execution time if one is sure that storage size will rapidly become 
			-- really bigger than `Default_size'.
			-- When first `remove' occurs, storage size may naturally become smaller than `medium_size'. 
			-- Afterall, tuning of storage size is done automatically according to usage.
		require
			medium_size > 0
		local
			new_capacity: INTEGER; hts: HASH_TABLE_SIZE
		do
			new_capacity := hts.prime_number_ceiling(medium_size)
			key_buckets := key_buckets.calloc(new_capacity)
			val_buckets := val_buckets.calloc(new_capacity)
			capacity := new_capacity
			cache_user := -1
			count := 0
		ensure
			is_empty
			capacity >= medium_size
		end

	key_safe_equal (k1, k2: K): BOOLEAN is
			-- Because keys are never Void, we do not rely on the SAFE_EQUAL class.
		require
			k1 /= Void
			k2 /= Void
		do
			if k1 = k2 then
				Result := True
			elseif k1.same_dynamic_type(k2) then
				Result := k1.is_equal(k2)
			end
		end
	
	val_safe_equal (v1, v2: V): BOOLEAN is
			-- Because values are never Void, we do not rely on the SAFE_EQUAL class.
		require
			v1 /= Void
			v2 /= Void
		do
			if v1 = v2 then
				Result := True
			elseif v1.same_dynamic_type(v2) then
				Result := v1.is_equal(v2)
			end
		end

	new_node (v: V; nv: HASHED_BIJECTIVE_DICTIONARY_NODE[V, K]
		       k: K; nk: HASHED_BIJECTIVE_DICTIONARY_NODE[V, K]): HASHED_BIJECTIVE_DICTIONARY_NODE[V, K] is
			-- Recycle from `free_nodes' or create a new one.
		do
			-- *** Recycling not yet implemented ****
			create Result.make(v, nv, k, nk)
		end
	
	increase_capacity is
			-- There is no more free slots: the dictionary must grow.
		require
			capacity = count
		local
			i, idx, new_capacity: INTEGER; old_key_buckets: like key_buckets; node1, node2: like cache_node
			hts: HASH_TABLE_SIZE; old_val_buckets: like val_buckets
		do
			new_capacity := hts.prime_number_ceiling(capacity + 1)
			-- Rebuilding `key_buckets':
			from
				old_key_buckets := key_buckets
				key_buckets := key_buckets.calloc(new_capacity)
				i := capacity - 1
			until
				i < 0
			loop
				from
					node1 := old_key_buckets.item(i)
				until
					node1 = Void
				loop
					node2 := node1.next_key
					idx := node1.key.hash_code #\\ new_capacity
					node1.set_next_key(key_buckets.item(idx))
					key_buckets.put(node1, idx)
					node1 := node2
				end
				i := i - 1
			end
			-- Rebuilding `val_buckets':
			from
				old_val_buckets := val_buckets
				val_buckets := val_buckets.calloc(new_capacity)
				i := capacity - 1
			until
				i < 0
			loop
				from
					node1 := old_val_buckets.item(i)
				until
					node1 = Void
				loop
					node2 := node1.next_val
					idx := node1.val.hash_code #\\ new_capacity
					node1.set_next_val(val_buckets.item(idx))
					val_buckets.put(node1, idx)
					node1 := node2
				end
				i := i - 1
			end
			-- Finally:
			capacity := new_capacity
			cache_user := -1
		ensure
			count = old count
			capacity > old capacity
		end

invariant
	capacity >= count

end
