-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class LIKE_ARGUMENT_WRAPPER
	--
	-- Used to wrap actual EXPRESSIONs which under the scope of a "like <argument>" type mark facility.
	--
	-- Because the "like <argument>" type mark is supposed to become obsolete, only feature `equal' and `clone' are handled.
	--

inherit
	EXPRESSION

creation {CALL_1}
	make

feature {ANY}
	actual_expression: EXPRESSION
			-- The one which is actually wrapped.

	type_expression: EXPRESSION
			-- The one used to determine the static type of the `actual_expression'.

	start_position: POSITION is
		do
			Result := actual_expression.start_position
		end

	is_manifest_string: BOOLEAN is
		do
			Result := actual_expression.is_manifest_string
		end

	is_static: BOOLEAN is
		do
			Result := actual_expression.is_static
		end

	is_void: BOOLEAN is
		do
			Result := actual_expression.is_void
		end

	is_result: BOOLEAN is
		do
			Result := actual_expression.is_result
		end

	is_current: BOOLEAN is
		do
			Result := actual_expression.is_current
		end

	is_implicit_current: BOOLEAN is
		do
			Result := actual_expression.is_implicit_current
		end

	declaration_type: TYPE is
		do
			Result := type_expression.declaration_type
		end

	has_been_specialized: BOOLEAN is
		do
			Result := actual_expression.has_been_specialized
		end

	adapt_for (type: TYPE): like Current is
		local
			ae: like actual_expression
		do
			ae := actual_expression.adapt_for(type)
			Result := current_or_twin_init(ae)
		end

	resolve_in (type: TYPE): TYPE is
		do
			Result := type_expression.resolve_in(type)
		end

	collect (type: TYPE): TYPE is
		do
			Result := actual_expression.collect(type)
			Result := type_expression.resolve_in(type)
		end

	specialize_2 (type: TYPE): EXPRESSION is
		local
			ae: like actual_expression
		do
			ae := actual_expression.specialize_2(type)
			Result := current_or_twin_init(ae)
		end

	specialize_in (type: TYPE): like Current is
		local
			ae: like actual_expression
		do
			ae := actual_expression.specialize_in(type)
			Result := current_or_twin_init(ae)
		end

	safety_check (type: TYPE) is
		do
			actual_expression.safety_check(type)
		end

	non_void_no_dispatch_type (type: TYPE): TYPE is
		do
		end

	simplify (type: TYPE): EXPRESSION is
		do
			Result := Current
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			ae: like actual_expression
		do
			ae := actual_expression.specialize_thru(parent_type, inherit_item, new_type)
			Result := current_or_twin_init(ae)
		end

	use_current (type: TYPE): BOOLEAN is
		do
			Result := actual_expression.use_current(type)
		end

	is_writable: BOOLEAN is
		do
			Result := actual_expression.is_writable
		end

	side_effect_free (type: TYPE): BOOLEAN is
		do
			Result := actual_expression.side_effect_free(type)
		end

	mapping_c_arg (type: TYPE; formal_type: TYPE_MARK) is
		do
			actual_expression.mapping_c_arg(type, formal_type)
		end

	mapping_c_target (type: TYPE; formal_type: TYPE_MARK) is
		do
			actual_expression.mapping_c_target(type, formal_type)
		end

	compile_to_c (type: TYPE) is
		do
			actual_expression.compile_to_c(type)
		end

	accept (visitor: LIKE_ARGUMENT_WRAPPER_VISITOR) is
		do
			visitor.visit_like_argument_wrapper(Current)
		end

	short (type: TYPE) is
		do
			actual_expression.short(type)
		end

	short_target (type: TYPE) is
		do
			actual_expression.short_target(type)
		end

	precedence: INTEGER is
		do
			Result := actual_expression.precedence
		end

	extra_bracket_flag: BOOLEAN is
		do
			Result := actual_expression.extra_bracket_flag
		end

	pretty (indent_level: INTEGER) is
		do
			actual_expression.pretty(indent_level)
		end

	pretty_target (indent_level: INTEGER) is
		do
			actual_expression.pretty_target(indent_level)
		end

	bracketed_pretty (indent_level: INTEGER) is
		do
			actual_expression.bracketed_pretty(indent_level)
		end

	compile_to_jvm (type: TYPE) is
		do
			actual_expression.compile_to_jvm(type)
		end

	compile_target_to_jvm (type: TYPE) is
		do
			actual_expression.compile_target_to_jvm(type)
		end

	jvm_branch_if_false (type: TYPE): INTEGER is
		do
			Result := actual_expression.jvm_branch_if_false(type)
		end

	jvm_branch_if_true (type: TYPE): INTEGER is
		do
			Result := actual_expression.jvm_branch_if_true(type)
		end

	jvm_assign_creation (type: TYPE) is
		do
			actual_expression.jvm_assign_creation(type)
		end

	jvm_assign (type: TYPE) is
		do
			actual_expression.jvm_assign(type)
		end

feature {LIKE_ARGUMENT_WRAPPER}
	init (ae: like actual_expression) is
		require
			ae /= Void
		do
			actual_expression := ae
		ensure
			actual_expression = ae
		end

feature {}
	make (ae: like actual_expression; te: like type_expression) is
		require
			ae /= Void
			te /= Void
		do
			actual_expression := ae
			type_expression := te
		ensure
			actual_expression = ae
			type_expression = te
		end

	current_or_twin_init (ae: like actual_expression): like Current is
		require
			ae /= Void
		do
			if ae = actual_expression then
				Result := Current
			else
				Result := twin
				Result.init(ae)
			end
		ensure
			Result.actual_expression = ae
			Result.type_expression = type_expression
		end

end -- class LIKE_ARGUMENT_WRAPPER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
