-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class MANIFEST_ARRAY
	--
	-- Like:  << foo , bar >>
	--
	--***PH: I think 'smallest_ancestor' is used too much

inherit
	EXPRESSION

creation {ANY}
	make

feature {ANY}
	start_position: POSITION
			-- Of first character of the first < character.

	is_current: BOOLEAN is False

	is_implicit_current: BOOLEAN is False

	is_writable: BOOLEAN is False

	is_manifest_string: BOOLEAN is False

	is_result: BOOLEAN is False

	is_void: BOOLEAN is False

	extra_bracket_flag: BOOLEAN is True

	precedence: INTEGER is 2

	smallest_ancestor (sp: POSITION; t1: TYPE; t2: TYPE): TYPE is
		do
			Result := t1.smallest_ancestor(t2)
			if Result = Void then
				error_handler.add_position(sp)
				error_handler.append("There is no common conformant parent for ")
				error_handler.append(t1.name.to_string)
				error_handler.append(" and ")
				error_handler.append(t2.name.to_string)
				error_handler.append(". Cannot compute the manifest array type.")
				error_handler.print_as_fatal_error
			end
		end

	side_effect_free (type: TYPE): BOOLEAN is
		do
			Result := False
		end

	is_static: BOOLEAN is False

	safety_check (type: TYPE) is
		local
			i: INTEGER
		do
			from
				i := list.upper
			until
				i < list.lower
			loop
				list.item(i).safety_check(type)
				i := i - 1
			end
		end

	mapping_c_target (type: TYPE; target_type: TYPE_MARK) is
		do
			cpp.put_character('(')
			target_type.mapping_cast
			compile_to_c(type)
			cpp.put_character(')')
		end

	mapping_c_arg (type: TYPE; formal_arg_type: TYPE_MARK) is
		do
			compile_to_c(type)
		end

	compile_to_c (type: TYPE) is
		local
			i: INTEGER; e: EXPRESSION; result_type: TYPE_MARK; ic: INTEGER_CONSTANT
		do
			result_type := resolve_in(type).canonical_type_mark
			manifest_array_pool.c_call(result_type)
			cpp.put_character('(')
			cpp.put_integer(list.count)
			from
				i := list.lower
			until
				i > list.upper
			loop
				cpp.put_string(once ",%N")
				e := list.item(i)
				if user_expanded_flag = Void then
					if ic ?:= e then
                  ic ::= e
						ic.compile_to_c(type)
					else
						e.compile_to_c(type)
					end
				else
					e.mapping_c_arg(type, user_expanded_flag)
				end
				i := i + 1
			end
			cpp.put_character(')')
		end

	compile_target_to_jvm, compile_to_jvm (type: TYPE) is
		local
			rt, elt_type: TYPE_MARK; i, idx: INTEGER; rc: LIVE_TYPE; idx_rc: INTEGER
			cp: like constant_pool; ca: like code_attribute
		do
			cp := constant_pool
			ca := code_attribute
			rt := resolve_in(type).canonical_type_mark
			rc := rt.live_type
			elt_type := rt.generic_list.first
			idx_rc := rc.jvm_constant_pool_index
			rc.jvm_basic_new
			-- Set lower:
			idx := cp.idx_fieldref4(idx_rc, as_lower, fz_i)
			ca.opcode_dup
			ca.opcode_iconst_1
			ca.opcode_putfield(idx, -2)
			-- Set upper:
			idx := cp.idx_fieldref4(idx_rc, as_upper, fz_i)
			ca.opcode_dup
			ca.opcode_push_integer(list.count)
			ca.opcode_putfield(idx, -2)
			-- Set capacity:
			idx := cp.idx_fieldref4(idx_rc, as_capacity, fz_i)
			ca.opcode_dup
			ca.opcode_push_integer(list.count)
			ca.opcode_putfield(idx, -2)
			-- Set storage:
			idx := cp.idx_fieldref4(idx_rc, as_storage, sd(elt_type))
			ca.opcode_dup
			ca.opcode_push_integer(list.count)
			elt_type.jvm_xnewarray
			from
				i := list.lower
			until
				i > list.upper
			loop
				ca.opcode_dup
				ca.opcode_push_integer(i)
				list.item(i).compile_to_jvm(type)
				elt_type.jvm_xastore
				i := i + 1
			end
			ca.opcode_putfield(idx, -2)
		end

	accept (visitor: MANIFEST_ARRAY_VISITOR) is
		do
			visitor.visit_manifest_array(Current)
		end

	jvm_branch_if_false, jvm_branch_if_true (type: TYPE): INTEGER is
		do
			check
				False
			end
		end

	use_current (type: TYPE): BOOLEAN is
		local
			i: INTEGER
		do
			from
				i := list.upper
			until
				i < list.lower or else Result
			loop
				Result := list.item(i).use_current(type)
				i := i - 1
			end
		end

	specialize_in (type: TYPE): like Current is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			error_handler.add_position(start_position)
			error_handler.append("This obsolete manifest ARRAY creation will be removed soon. %
                              %You must now use the generic creation form (see %
										%SmartEiffel/tutorial/manifest_expression.e for details). %
                              %Update your code before release 2.2. (Oct 30th 2004)")
			error_handler.print_as_warning
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.specialize_in(type)
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).specialize_in(type), i)
					i := i - 1
				end
			end
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.specialize_thru(parent_type, inherit_item, new_type)
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).specialize_thru(parent_type, inherit_item, new_type), i)
					i := i - 1
				end
			end
		end

	specialize_2 (type: TYPE): like Current is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.specialize_2(type)
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).specialize_2(type), i)
					i := i - 1
				end
			end
			Result := Result.add_necessary_casts(type, Result = Current)
		end

	has_been_specialized: BOOLEAN is
		local
			i: INTEGER
		do
			Result := True
			from
				i := list.upper
			until
				i < list.lower or else not Result
			loop
				Result := list.item(i).has_been_specialized
				i := i - 1
			end
		end

	adapt_for (type: TYPE): like Current is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			Result := Current
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.adapt_for(type)
				i := i - 1
			end
			if exp1 /= exp2 then
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).adapt_for(type), i)
					i := i - 1
				end
			end
			Result.update_user_expanded_flag(type)
		end

	non_void_no_dispatch_type (type: TYPE): TYPE is
		do
			Result := resolve_in(type)
		end

	simplify (type: TYPE): MANIFEST_ARRAY is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.simplify(type)
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				smart_eiffel.magic_count_increment
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).simplify(type), i)
					i := i - 1
				end
			end
		end

	bracketed_pretty (indent_level: INTEGER) is
		do
			pretty_printer.put_character('(')
			pretty(indent_level)
			pretty_printer.put_character(')')
		end

	pretty (indent_level: INTEGER) is
		local
			i: INTEGER
		do
			pretty_printer.set_indent_level(indent_level)
			pretty_printer.put_string(once "<<")
			from
				i := list.lower
			until
				i > list.upper
			loop
				list.item(i).pretty(indent_level + 1)
				i := i + 1
				if i <= list.upper then
					pretty_printer.put_string(once ", ")
					if pretty_printer.column > pretty_printer.wrap_limit then
						pretty_printer.set_indent_level(indent_level + 1)
					end
				end
			end
			pretty_printer.put_string(once ">>")
		end

	pretty_target (indent_level: INTEGER) is
		do
			pretty_printer.put_character('(')
			pretty(indent_level)
			pretty_printer.put_character(')')
			pretty_printer.put_character('.')
		end

	short (type: TYPE) is
		local
			i: INTEGER
		do
			short_printer.hook_or("op_ma", fz_c_shift_left)
			from
				i := list.lower
			until
				i > list.upper
			loop
				list.item(i).short(type)
				i := i + 1
				if i <= list.upper then
					short_printer.hook_or("ma_sep", ",")
				end
			end
			short_printer.hook_or("cl_ma", fz_c_shift_right)
		end

	short_target (type: TYPE) is
		do
			bracketed_short(type)
			short_printer.put_dot
		end

	jvm_assign_creation, jvm_assign (type: TYPE) is
		do
			check
				False
			end
		end

feature {ANY}
	declaration_type: TYPE is
		local
			i: INTEGER; sp: POSITION; most_general_type, t2: TYPE
		do
			Result := declaration_type_memory
			if Result = Void then
				most_general_type := list.last.declaration_type
				from
					i := list.upper - 1
				until
					i < list.lower
				loop
					t2 := list.item(i).declaration_type
					sp := list.item(i).start_position
					most_general_type := smallest_ancestor(sp, most_general_type, t2)
					i := i - 1
				end
				Result := smart_eiffel.get_array_type(most_general_type, start_position)
				declaration_type_memory := Result
			end
		end

	resolve_in (type: TYPE): TYPE is
		local
			i: INTEGER; sp: POSITION; most_general_type, t2: TYPE
		do
			most_general_type := list.last.resolve_in(type)
			from
				i := list.upper - 1
			until
				i < list.lower
			loop
				t2 := list.item(i).resolve_in(type)
				sp := list.item(i).start_position
				most_general_type := smallest_ancestor(sp, most_general_type, t2)
				i := i - 1
			end
			if most_general_type.is_none then
				error_handler.add_position(start_position)
				error_handler.append("An array denoted ")
				error_handler.add_expression(Current)
				error_handler.append(" is actually of type ARRAY[NONE] which means that it can holds only `Void' values! %
											%(If this is _really_ what you want, you can work arround by using an ordinary %
											%%"create%" expression / instruction.)")
				error_handler.print_as_fatal_error
			end
			Result := smart_eiffel.get_array_type(most_general_type, start_position)
		end

	collect (type: TYPE): TYPE is
		local
			i: INTEGER; expression: EXPRESSION; elt_type, most_general_type: TYPE; tmp: TYPE
		do
			most_general_type := list.last.collect(type)
			from
				i := list.upper - 1
			until
				i < list.lower
			loop
				elt_type := list.item(i).collect(type)
				most_general_type := smallest_ancestor(list.item(i).start_position, most_general_type, elt_type)
				i := i - 1
			end
			-- Now notifying the `assignment_handler':
			from
				i := list.upper
			until
				i < list.lower
			loop
				expression := list.item(i)
				elt_type := expression.collect(type)
				assignment_handler.collect_normal(elt_type, most_general_type)
				i := i - 1
			end
			Result := smart_eiffel.get_array_type(most_general_type, start_position) -- Make the type as alive
			tmp := smart_eiffel.collect(Result, Result.feature_stamp_of(make_name), True)
			manifest_array_pool.register(Result.canonical_type_mark)
		end

feature {MANIFEST_ARRAY}
	add_necessary_casts (type: TYPE; can_twin: BOOLEAN): like Current is
			-- This function has been made because the type of the array
			-- is not known at the begining of specialize_2. The
			-- smallest ancestor can only be computed once specialize_2
			-- has been called on all elements, and this smallest
			-- ancestor is needed to determine if implicit_cast has to
			-- be added or not.
			-- If the type were known, then the best way were to do the
			-- work in specialize_2 in 3 steps for each element: call
			-- specialize_2, find the type, add implicit_cast if needed.
		require
		-- specialize_2 has been done (needed to call resolve_in)
		local
			exp1, exp2: EXPRESSION; generic_type, exp1_type: TYPE; l: like list; i: INTEGER
		do
			generic_type := resolve_in(type).generic_list.first
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp1_type := exp1.resolve_in(type)
				if exp1_type = generic_type then
					exp2 := exp1
				else
					exp2 := assignment_handler.implicit_cast(exp1, exp1_type, generic_type)
				end
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				from
					if can_twin then
						l := list.twin
						Result := twin
						Result.set_list(l)
					else
						l := list
						Result := Current
					end
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					exp1 := list.item(i)
					exp1_type := exp1.resolve_in(type)
					if exp1_type = generic_type then
						exp2 := exp1
					else
						exp2 := assignment_handler.implicit_cast(exp1, exp1_type, generic_type)
					end
					l.put(exp2, i)
					i := i - 1
				end
			end
		ensure
			not can_twin implies Result = Current
		end

	set_list (l: like list) is
		do
			list := l
		end

	update_user_expanded_flag (type: TYPE) is
		local
			generic_type: TYPE
		do
			generic_type := resolve_in(type).generic_list.first
			if generic_type.is_user_expanded then
				if not generic_type.is_dummy_expanded then
					user_expanded_flag := generic_type.canonical_type_mark
				end
			end
		end

feature {MANIFEST_ARRAY_VISITOR}
	list: FAST_ARRAY[EXPRESSION]
			-- Void or elements in the manifest array.

feature {}
	declaration_type_memory: TYPE

	user_expanded_flag: TYPE_MARK

	make (sp: like start_position; l: like list) is
		require
			not sp.is_unknown
			l /= Void implies not l.is_empty
		do
			start_position := sp
			if l = Void then
				error_handler.add_position(sp)
				error_handler.append("Empty manifest array not allowed. (If you really need to do it, just replace it %
                                 %with something like:%Ncreate {ARRAY[ANY]}.make(1, 0)")
				error_handler.print_as_fatal_error
			end
			list := l
		ensure
			start_position = sp
			list = l
		end

	sd (elt_type: TYPE_MARK): STRING is
			-- The JVM descriptor for `storage'.
		do
			sd_buffer.clear_count
			sd_buffer.extend('[')
			elt_type.jvm_descriptor_in(sd_buffer)
			Result := sd_buffer
		end

	sd_buffer: STRING is
		once
			create Result.make(16)
		end

invariant
	not start_position.is_unknown

	not list.is_empty

end -- class MANIFEST_ARRAY
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
