-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
deferred class EXTERNAL_ROUTINE
	--
	-- For routines implemented with a call to a foreign language.
	-- Root of EXTERNAL_PROCEDURE and EXTERNAL_FUNCTION.
	--

inherit
	E_ROUTINE

feature {ANY}
	native: NATIVE

	alias_string: MANIFEST_STRING

	is_deferred: BOOLEAN is False

	frozen rescue_compound: COMPOUND is
		do
		end

	frozen set_rescue_compound (instruction: INSTRUCTION) is
		do
			if instruction /= Void then
				error_handler.add_position(instruction.start_position)
			else
				error_handler.add_position(start_position)
			end
			error_handler.append("External feature must not have rescue compound.")
			error_handler.print_as_fatal_error
		end

	frozen use_current (type: TYPE): BOOLEAN is
		do
			Result := native.use_current(Current)
		end

	external_name: STRING is
		local
			native_c_plus_plus: NATIVE_C_PLUS_PLUS; tmp: STRING
		do
			native_c_plus_plus ?= native
			if native_c_plus_plus = Void then
				Result := c_plus_plus_name
			else
				-- For C++, there is always a wrapping function :
				tmp := first_name.to_string
				create Result.make(tmp.count + 8)
				Result.append(once "cpp")
				class_text.id.append_in(Result)
				Result.append(tmp)
			end
		end

	specialize_body_in (new_type: TYPE; can_twin: BOOLEAN): like Current is
		do
			--*** body have to be specialized
			Result := Current
		end

	specialize_body_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE; can_twin: BOOLEAN): like Current is
		do
			--*** body have to be specialized
			Result := Current
		end

	specialize_2 (type: TYPE): like Current is
		local
			ra: like require_assertion; ea: like ensure_assertion
		do
			if ace.boost then
				Result := Current
			else
				if require_assertion /= Void then
					ra := require_assertion.specialize_2(type)
				end
				if ensure_assertion /= Void then
					ea := ensure_assertion.specialize_2(type)
				end
				if ra = require_assertion and then ea = ensure_assertion then
					Result := Current
				else
					Result := twin
					Result.set_require(ra)
					Result.set_ensure(ea)
				end
			end
		end

	has_been_specialized: BOOLEAN is
		do
			if ace.boost then
				Result := True
			else
				Result := require_assertion /= Void implies require_assertion.has_been_specialized
				if Result and then ensure_assertion /= Void then
					Result := ensure_assertion.has_been_specialized
				end
			end
		end

feature {FEATURE_STAMP, LIVE_TYPE, PRECURSOR_CALL}
	simplify (type: TYPE): like Current is
		do
			Result := Current
		end

feature {NATIVE_C}
	argument_rank_of (arg_name: STRING): INTEGER is
			-- Gives 0 or the index of `arg_name'.
		require
			string_aliaser.registered_one(arg_name)
		do
			if arguments /= Void then
				Result := arguments.rank_of(arg_name)
			end
		ensure
			arguments /= Void implies Result.in_range(0, arguments.count)
			arguments = Void implies Result = 0
		end

feature {NATIVE_C_PLUS_PLUS}
	c_plus_plus_name: STRING is
			-- The name of the corresponding member in C++ is the
			-- `alias_string' if any or the `first_name' itself.
		do
			if alias_string = Void then
				Result := first_name.to_string
			else
				Result := alias_string.to_string
			end
		end

feature {}
	add_into_ (ft: like feature_text; fd: DICTIONARY[ANONYMOUS_FEATURE, FEATURE_NAME]) is
		do
			add_into_shared(ft, fd)
		end

	pretty_print_routine_body is
		do
			pretty_printer.set_indent_level(2)
			pretty_printer.keyword(once "external")
			native.pretty
			if alias_string /= Void then
				pretty_printer.set_indent_level(2)
				pretty_printer.keyword(once "alias")
				alias_string.pretty(3)
			end
			pretty_printer.set_indent_level(0)
		end

	pretty_print_rescue is
		do
		end

	make_external_routine (n: like native; description: like alias_string) is
		require
			n /= Void
		do
			native := n
			alias_string := description
		end

end -- class EXTERNAL_ROUTINE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
