-- See the Copyright notice at the end of this file.
--
class HASHED_BIJECTIVE_DICTIONARY_NODE[V -> HASHABLE, K -> HASHABLE]
	--
	-- Auxilliary class to implement HASHED_BIJECTIVE_DICTIONARY[V -> HASHABLE, K -> HASHABLE].
	--

creation {BIJECTIVE_DICTIONARY}
	make

feature {BIJECTIVE_DICTIONARY}
	val: V

	key: K

	next_key: like Current
			-- The forward link to the next `key' in case of hash-code clash.

	next_val: like Current
			-- The forward link to the next `val' in case of hash-code clash.

	make (v: like val; nv: like next_val; k: like key; nk: like next_key) is
		require
			v /= Void
			k /= Void
		do
			val := v
			next_val := nv
			key := k
			next_key := nk
		ensure
			val = v
			next_val = nv
			key = k
			next_key = nk
		end
	
	set_val (v: like val) is
		do
			val := v
		ensure
			val = v
		end
	
	set_next_val (nv: like next_val) is
		do
			next_val := nv
		ensure
			next_val = nv
		end

	set_key (k: like key) is
		do
			key := k
		ensure
			key = k
		end
	
	set_next_key (nk: like next_key) is
		do
			next_key := nk
		ensure
			next_key = nk
		end

invariant
	key /= Void

	val /= Void
	
end -- class HASHED_BIJECTIVE_DICTIONARY_NODE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
