-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class CREATE_INSTRUCTION
	--
	-- The new style create instruction.
	--

inherit
	INSTRUCTION

insert
	CREATE_TOOLS

creation {EIFFEL_PARSER}
	make

feature {ANY}
	specialize_in (new_type: TYPE): like Current is
		local
			et: like explicit_type; w: like writable; c: like call
		do
			if explicit_type /= Void then
				et := explicit_type.specialize_in(new_type)
			end
			w := writable.specialize_in(new_type)
			if call /= Void then
				c := call.specialize_in(new_type)
			end
			Result := current_or_twin_init(et, w, c)
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			et: like explicit_type; w: like writable; c: like call
		do
			if explicit_type /= Void then
				et := explicit_type.specialize_thru(parent_type, inherit_item, new_type)
			end
			w := writable.specialize_thru(parent_type, inherit_item, new_type)
			if call /= Void then
				c := call.specialize_thru(parent_type, inherit_item, new_type)
			end
			Result := current_or_twin_init(et, w, c)
		end

	specialize_2 (type: TYPE): like Current is
		local
			w: like writable; c: like call; tm: TYPE_MARK; creation_type: TYPE; fs: FEATURE_STAMP
			name: FEATURE_NAME; test_default: BOOLEAN
		do
			w := writable.specialize_2(type)
			-- To check/replace create {DICTIONARY} with create {HASHED_DICTIONARY} (or SET...), creation_type 
			-- is computed:
			if explicit_type /= Void then
				tm := explicit_type.to_static(type)
				creation_type := tm.type
			else
				creation_type := w.resolve_in(type)
				tm := creation_type.canonical_type_mark
			end
			tm := smart_eiffel.deferred_dictionary_or_set_creation_check(start_position, tm)
			if tm /= Void then
				create {CREATE_WRITABLE} w.make_specialized(tm, w)
				explicit_type := tm
				if call /= Void then
					call.set_target(w)
				end
				creation_type := explicit_type.resolve_in(type)
			end
			-- check/replace warning done.
			if call = Void then
				-- Create the call to the default creation feature if it is implicit.
				default_create_call := True
				fs := general_default_create_fs.resolve_dynamic_binding_for(creation_type)
				create name.ordinary_name(creation_type.get_feature_name(fs).name, w.start_position)
				create {PROC_CALL_0} c.make(w, name)
				if explicit_type /= Void then
					c.set_target(w)
				end
			else
				c := call
				test_default := True
			end
			c.set_creation_type(creation_type.canonical_type_mark)
			c ::= c.specialize_2(type)
			check
				c /= Void
			end
			if test_default then
				fs := general_default_create_fs.resolve_dynamic_binding_for(creation_type)
				default_create_call := c.feature_stamp = fs
			end
			Result := current_or_twin_init(explicit_type, w, c)
			Result.specialize_2_checks(type)
		ensure then
			Result.call /= Void
		end

	has_been_specialized: BOOLEAN is
		do
			Result := writable.has_been_specialized and then (call /= Void implies call.has_been_specialized)
		end

	collect (type: TYPE) is
		local
			writable_type: TYPE; lt: LIVE_TYPE; type_to_create: TYPE
		do
			writable_type := writable.collect(type)
			if call /= Void then
				call.collect_create(type, writable_type, writable.start_position)
			else
				if explicit_type = Void then
					type_to_create := writable_type
				else
					type_to_create := explicit_type.resolve_in(type)
				end
				lt := smart_eiffel.collect_one_type(type_to_create, True)
			end
		end

	adapt_for (type: TYPE): like Current is
		local
			w: like writable; c: like call; et: like explicit_type
		do
			w := writable.adapt_for(type)
			if call /= Void then
				c := call.adapt_for(type)
			end
			if explicit_type /= Void then
				et := explicit_type.to_static(type)
			end
			Result := current_or_twin_init(et, w, c)
		end

	frozen use_current (type: TYPE): BOOLEAN is
		local
			args: like arguments
		do
			if writable.use_current(type) then
				Result := True
			else
				args := arguments
				if args /= Void then
					Result := args.use_current(type)
				end
			end
		end

	simplify (type: TYPE): INSTRUCTION is
		local
			c: like call; creation_proc: E_PROCEDURE; args: EFFECTIVE_ARG_LIST; type_to_create: TYPE
		do
			if call /= Void then
				c := call.simplify_arguments(type)
				if explicit_type = Void then
					type_to_create := writable.resolve_in(type)
				else
					type_to_create := explicit_type.resolve_in(type)
				end
				creation_proc ?= c.feature_stamp.anonymous_feature(type_to_create)
				if creation_proc.routine_body = Void then
					args := c.arguments
					if args = Void or else args.side_effect_free(type) then
						c := Void
						smart_eiffel.magic_count_increment
					end
				end	
			end
			Result := current_or_twin_init(explicit_type, writable, c)
			if Result /= Current then
				smart_eiffel.magic_count_increment
			end
		end

	pretty (indent_level: INTEGER) is
		do
			pretty_printer.set_indent_level(indent_level)
			pretty_printer.keyword(once "create")
			if explicit_type /= Void then
				pretty_printer.put_character('{')
				pretty_printer.put_type_mark(explicit_type)
				pretty_printer.put_character('}')
				pretty_printer.put_character(' ')
			end
			writable.pretty(indent_level)
			if call /= Void then
				pretty_printer.put_character('.')
				pretty_printer.put_string(call.feature_name.to_string)
				if call.arguments /= Void then
					call.arguments.pretty(indent_level)
				end
			end
			if pretty_printer.semi_colon_flag then
				pretty_printer.put_character(';')
			end
			pretty_printer.set_indent_level(0)
		end

	frozen compile_to_c (type: TYPE) is
			--*** @TODO: lock till the end of the procedure call
		local
			created: TYPE_MARK; rf: RUN_FEATURE; rf3: RUN_FEATURE_3
		do
			created := explicit_type
			if created = Void then
				created := writable.resolve_in(type).canonical_type_mark
			end
			if created.is_reference then
				c2c_opening(type, created)
				if call /= Void then
					rf := call.run_feature_for(type)
					cpp.push_create_instruction(type, rf, arguments)
					rf.mapping_c
					cpp.pop
				end
				--***if smart_eiffel.scoop and then t.is_separate then
				--	 cpp.put_string(once "se_join_subsystem(n->subsystem);%N")
				--end
				c2c_closing(type, created)
			else
				writable.compile_to_c(type)
				cpp.put_string(once "=M")
				cpp.put_integer(created.id)
				cpp.put_string(once ";%N")
				rf := created.live_type.default_create_run_feature
				if rf /= Void then
					rf3 ::= rf
					cpp.put_proc_call_0(type, rf3, writable, Void)
				end
			end
		end

	frozen compile_to_jvm (type: TYPE) is
		local
			w: EXPRESSION; created: TYPE_MARK; rf: RUN_FEATURE
		do
			w := writable
			if explicit_type /= Void then
				created := explicit_type
			else
				created := writable.resolve_in(type).canonical_type_mark
			end
			compile_to_jvm0(created)
			if call /= Void then
				rf := call.run_feature_for(type)
			elseif created.is_user_expanded then
				rf := created.live_type.default_create_run_feature
			end
			if rf /= Void then
				jvm.inside_create_instruction(type, rf, call)
			end
			created.jvm_check_class_invariant
			w.jvm_assign_creation(type)
		end

	end_mark_comment: BOOLEAN is False

	accept (visitor: CREATE_INSTRUCTION_VISITOR) is
		do
			visitor.visit_create_instruction(Current)
		end

	verify_scoop (type: TYPE; allowed: FORMAL_ARG_LIST) is
		do
		end

feature {CREATE_INSTRUCTION}
	init (et: like explicit_type; w: like writable; c: like call) is
		require
			w /= Void
		do
			explicit_type := et
			writable := w
			call := c
		ensure
			explicit_type = et
			writable = w
			call = c
		end

feature {}
	c2c_opening (type: TYPE; t: TYPE_MARK) is
		require
			t.is_reference
		local
			lt: LIVE_TYPE; e_result: E_RESULT
		do
			lt := t.live_type
			if ace.no_check then
				cpp.put_trace_or_sedb_instruction(start_position)
			end
			cpp.put_character('{')
			gc_handler.declare_allocate_n(lt)
			cpp.expanded_attributes(type, t)
			e_result ?= writable
			if e_result /= Void and then e_result.is_once_result then
				cpp.put_string(e_result.c_variable_name)
				cpp.put_string(once "=((T0*)n);%N")
			end
		end

	c2c_closing (type: TYPE; t: TYPE_MARK) is
		require
			t.is_reference
		local
			e_result: E_RESULT
		do
			e_result ?= writable
			if e_result = Void or else not e_result.is_once_result then
				writable.compile_to_c(type)
				cpp.put_string(once "=((T0*)n);%N")
			end
			if cpp.call_invariant_start(t) then
				cpp.put_character('n')
				cpp.call_invariant_end
				cpp.put_character(';')
			end
			cpp.put_character('}')
			cpp.put_character('%N')
		end

	make (sp: like start_position; et: like explicit_type; w: like writable; c: like call) is
		require
			not sp.is_unknown
			w /= Void
			c /= Void implies c.target = w
		do
			start_position := sp
			call := c
			if et = Void then
				writable := w
			else
				create {CREATE_WRITABLE} writable.make(et, w)
				explicit_type := et
				if call /= Void then
					call.set_target(writable)
				end
			end
		ensure
			start_position = sp
			explicit_type = et
			writable /= Void
			call = c
		end

	current_or_twin_init (et: like explicit_type; w: like writable; c: like call): like Current is
		require
			w /= Void
		do
			if et = explicit_type and then w = writable and then c = call then
				Result := Current
			else
				Result := twin
				Result.init(et, w, c)
			end
		ensure
			Result.explicit_type = et
			Result.writable = w
			Result.call = c
		end

invariant
	not start_position.is_unknown

	writable.is_writable

end -- class CREATE_INSTRUCTION
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
