-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class LIKE_ARGUMENT_TYPE_MARK
	--
	-- For an anchored declaration type mark using a formal argument.
	--
	-- See also TYPE_LIKE and TYPE_LIKE_CURRENT.
	--

inherit
	ANCHORED_TYPE_MARK

creation {EIFFEL_PARSER, FORMAL_ARG_LIST}
	make

feature {ANY}
	like_what: ARGUMENT_NAME2

	written_name: HASHED_STRING

	has_been_specialized: BOOLEAN is True

	frozen specialize_in (new_type: TYPE): like Current is
		do
			if not like_what.formal_arg_list.type_mark(rank).is_static then
				error_handler.append(once "Sorry, SmartEiffel can currently handle types that are anchored to %
												  %a routine's argument only if this argument does not involve anchored %
												  %types or formal generic types.")
				error_handler.add_position(start_position)
				error_handler.add_position(like_what.formal_arg_list.type_mark(rank).start_position)
				error_handler.print_as_fatal_error
			end
			Result := Current
		end

	frozen specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		do
			Result := Current
		end
		--   declaration_type: like Current is
		--      do
		--	 Result := Current
		--      end

	declaration_type: TYPE_MARK is
			--***PH: this definition allow to run tests even if
			--LIKE_ARGUMENT_TYPE_MARK is broken. It's not necessary for our
			--own code ;-) so it will wait...
		do
			Result := like_what.formal_arg_list.type_mark(rank).declaration_type
		end

	rank: INTEGER is
		do
			Result := like_what.rank
		end

	accept (visitor: LIKE_ARGUMENT_TYPE_MARK_VISITOR) is
		do
			visitor.visit_like_argument_type_mark(Current)
		end

	to_static (t: TYPE): TYPE_MARK is
		do
			if like_argument_computing then
				error_handler.append("Unable to solve cyclic anchored types.")
				like_argument_computing := False
			else
				like_argument_computing := True
				Result := like_what.formal_arg_list.type_mark(rank).to_static(t)
				if Result = Void then
					error_handler.add_position(start_position)
				end
				if like_argument_computing = False then
					error_handler.print_as_fatal_error
				end
				like_argument_computing := False
			end
		end

	signature_resolve_in (new_type: TYPE): TYPE is
		do
			Result := like_what.formal_arg_list.type_mark(rank).signature_resolve_in(new_type)
		end

feature {}
	like_argument_computing: BOOLEAN

feature {TYPE_MARK}
	short_ (shorted_type: TYPE) is
		do
			short_printer.hook_or(once "like", once "like ")
			like_what.short(shorted_type)
		end

feature {}
	make (sp: like start_position; lw: like like_what) is
		require
			not sp.is_unknown
			lw /= Void
		do
			start_position := sp
			like_what := lw
			written_mark_buffer.copy(fz_like)
			written_mark_buffer.extend(' ')
			written_mark_buffer.append(like_what.to_string)
			written_name := string_aliaser.hashed_string(written_mark_buffer)
		ensure
			start_position = sp
			like_what = lw
		end

invariant
	as_current /= like_what.to_string

end -- class LIKE_ARGUMENT_TYPE_MARK
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
