class SOCKETS
--
-- This is the first tutorial class to look at, because it gives the most simple examples.
--
creation {ANY}
   make

feature {}
   make is
      local
         tcp: TCP_ACCESS
			host: HOST
         ip_tcp: IP_ADDRESS
      do
			-- The example just tries to connect to a well-known ftp server

         create tcp.make(21)
         create host.make(once "ftp.funet.fi", tcp)
         test_socket(host)

			-- Now the following lines start a real server, listening on port 2001.
         create tcp.make(2001)
         create ip_tcp.make(127,0,0,1, tcp)
         test_server(ip_tcp)
      end

   test_socket (address: ADDRESS) is
         -- First tutorial: simple network connection
      local
         ios: SOCKET_INPUT_OUTPUT_STREAM
      do
         -- Create a stream connected to the address
         ios := address.stream
         if ios.is_connected then
            ios.read_line
            io.put_string(ios.last_string)
            io.put_new_line
            ios.disconnect
         end
      end

   test_server (address: ADDRESS) is
         -- Second tutorial: create a server. This one is trivial, see MULTIPLEX_SERVER for a real-life
         -- example
      local
         server: SOCKET_SERVER
      do
         -- About the stack and the sequencer: see the sequencer tutorial
         create stack.make
         -- Create a server listeneing at the given address
         server := address.server
         -- Must test if the server is effectively listening (otherwise there was an error: address in use and
         -- so on)
         if server.can_connect then
            -- Add a handler: the agent is called by the server when an incoming connection happens
            server.when_connect(agent handle_connect)
            -- Add the job to the sequencer stack
            stack.add_job(server)
            io.put_string(once "Accepting connections on port ")
            io.put_integer(address.access.port)
            io.put_new_line
            -- And we start the sequencer loop
            stack.run
         end
      end

feature {}
   stack: LOOP_STACK

   handle_connect (ios: SOCKET_INPUT_OUTPUT_STREAM) is
         -- Handle a connection to the server
      local
         stop: BOOLEAN
         s: STRING
      do
         from
            io.put_string(once "(new connection accepted)%N")
            ios.unset_timeout
         until
            stop
         loop
            ios.read_line
            s := ios.last_string
            if not s.is_empty then
               stop := s.same_as(once "stop")
               io.put_string(s)
               io.put_new_line
            end
         end
         ios.disconnect
         io.put_string(once "(connection cancelled)%N")
			-- This last line will stop the SERVER!
         stack.break
      end

end
